# -*- coding: utf-8 -*-

from .context import injection_context
from .generators import default_oid_generator
from .injectors import PrefixInjector
from .prefix import make_prefix, join_prefix


def direct_injector(prefix, container=None):
    injector = PrefixInjector(prefix)
    return injection_context(injector, container)


def prefix_injector(prefix, delimiter=None, container=None):
    prefix = make_prefix(prefix, delimiter)
    injector = PrefixInjector(prefix)
    return injection_context(injector, container)


def autoprefix_injector(oid_generator=None, delimiter=None, container=None):
    autopart = next(oid_generator or default_oid_generator)
    return prefix_injector(autopart, delimiter, container)


def hybrid_injector(prefix, oid_generator=None, delimiter=None,
                    container=None):
    autopart = next(oid_generator or default_oid_generator)
    prefix = join_prefix([autopart, prefix, ], delimiter)
    injector = PrefixInjector(prefix)
    return injection_context(injector, container)



# -*- coding: utf-8 -*-

import uuid

from .defaults import OID_LENGTH, OID_MAX_LENGTH


def generate_uuid_based_oid(length=None):
    """
    OID generator which uses uuid.uuid4 (random UUIDs) to produce result.
    """
    length = min(length or OID_LENGTH, OID_MAX_LENGTH)

    while True:
        yield uuid.uuid4().hex.upper()[:length]


generate_default_oid = generate_uuid_based_oid
default_oid_generator = generate_default_oid()



# -*- coding: utf-8 -*-

from .defaults import DELIMITER


def make_prefix(prefix, delimiter=None):
    return prefix + (delimiter or DELIMITER)


def join_prefix(chunks, delimiter=None):
    delimeter = delimiter or DELIMITER
    return make_prefix(delimeter.join(chunks), delimeter)



# -*- coding: utf-8 -*-

import os

from setuptools import setup


__here__ = os.path.abspath(os.path.dirname(__file__))

README = open(os.path.join(__here__, 'README.rst')).read()

# Get VERSION
version_file = os.path.join('isotopic_logging', 'version.py')
# Use exec for compabibility with Python 3
exec(open(version_file).read())

setup(
    name='isotopic-logging',
    version=globals()['VERSION'],
    description='Mark and trace events in your log alike isotopic labeling',
    long_description=README,
    keywords=[
        'library', 'logging',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    url='https://github.com/oblalex/isotopic-logging',
    author='Alexander Oblovatniy',
    author_email='oblovatniy@gmail.com',
    license='LGPLv3',
    packages=[
        'isotopic_logging',
    ],
    scripts=[
    ],
    platforms=[
        'any',
    ],
    include_package_data=False,
    install_requires=[
    ],
    zip_safe=False,
)



# -*- coding: utf-8 -*-

from contextlib import contextmanager
from threading import local


_default_container = local()


@contextmanager
def injection_context(injector, container=None):
    container = container or _default_container
    is_top_level_call = not hasattr(container, '_injector')

    if is_top_level_call:
        container._injector = injector

    try:
        yield container._injector
    except Exception as e:
        raise e
    finally:
        if is_top_level_call:
            del container._injector



# -*- coding: utf-8 -*-


class PrefixInjector(object):

    __slots__ = ['prefix', ]

    def __init__(self, prefix):
        self.prefix = prefix

    def mark(self, message):
        return self.prefix + message
